from rest_framework_simplejwt.views import TokenObtainPairView

from burl.core.api.serializers import TokenObtainUserPairSerializer


class TokenObtainUserPairView(TokenObtainPairView):
    serializer_class = TokenObtainUserPairSerializer

    def post(self, request, *args, **kwargs):
        response = super().post(request, *args, **kwargs)
        return response



from django.test import TestCase
from django.contrib.auth import get_user_model

import uuid


class BurlUserTests(TestCase):
    def setUp(self):
        self.user = get_user_model().objects.create_user("farnsworth")

    def test_user_uuid(self):
        self.assertIsInstance(self.user.id, uuid.UUID)



from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.urls import reverse
from rest_framework.test import APITestCase

from django_burl.models import BriefURL, BriefURLDomainUser


class BurlCoreApiTestCase(APITestCase):
    @classmethod
    def setUpTestData(cls):
        Site.objects.filter(id=1).update(domain="testserver", name="testserver")
        cls.site = Site.objects.get(id=1)
        cls.url = "https://en.wikipedia.org/"
        cls.amy = get_user_model().objects.create_user("amy", password="amy54321")
        BriefURLDomainUser.objects.create(
            site=cls.site, user=cls.amy, role=BriefURLDomainUser.Role.ADMIN
        )
        cls.burl = BriefURL.objects.create(
            url="https://slashdot.org", user=cls.amy, site=cls.site
        )


class BurlCoreApiTestJWT(BurlCoreApiTestCase):
    def test_get_jwt_token(self):
        data = {"username": "amy", "password": "amy54321"}
        response = self.client.post(reverse("api_v2:token_auth"), data, format="json")
        self.assertEquals(response.status_code, 200)
        self.assertTrue("access" in response.json().keys())
        self.assertTrue("user" in response.json().keys())
        self.assertTrue("refresh" in response.json().keys())

    def test_use_jwt_token(self):
        data = {"username": "amy", "password": "amy54321"}
        response = self.client.post(reverse("api_v2:token_auth"), data, format="json")
        token = response.json()["access"]
        self.client.credentials(HTTP_AUTHORIZATION=f"Bearer {token}")
        response = self.client.get(
            reverse("api_v2:burls:burls-detail", kwargs={"burl": self.burl.burl}),
        )
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.json()["burl"], self.burl.burl)
        self.assertEquals(response.json()["url"], self.burl.url)



from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.shortcuts import redirect

from django_burl.models import BriefURLDefaultRedirect


def root_redirect(request):
    redirect_to = settings.DEFAULT_REDIRECT_URL
    try:
        redirect_to = BriefURLDefaultRedirect.objects.get(site=request.site).url
    except ObjectDoesNotExist:
        pass
    return redirect(redirect_to)



from rest_framework_simplejwt.serializers import TokenObtainPairSerializer
from rest_framework_simplejwt.tokens import RefreshToken


class TokenObtainUserPairSerializer(TokenObtainPairSerializer):
    def update(self, instance, validated_data):
        return super().update(instance, validated_data)

    def create(self, validated_data):
        return super().create(validated_data)

    @classmethod
    def get_token(cls, user):
        return RefreshToken.for_user(user)

    def validate(self, attrs):
        data = super().validate(attrs)
        data["user"] = self.user.id
        return data



import os
from django.core.exceptions import ImproperlyConfigured


def get_env(setting, fallback=None):
    try:
        return os.environ[setting]
    except KeyError:
        if fallback:
            return fallback
        else:
            return ImproperlyConfigured(
                "Please set the {} environment variable".format(setting)
            )


def get_log_dir():
    if "HOME" in os.environ:
        home = os.environ["HOME"]
        try_dirs = [
            "/var/log/burl",
            os.path.join(home, ".local", "var", "log", "burl"),
            home,
            "/tmp/burl",
            "/tmp",
        ]
    else:
        try_dirs = ["/var/log/burl", "/tmp/burl", "/tmp"]
    for log_dir in try_dirs:
        if os.path.isdir(log_dir):
            if os.access(log_dir, os.W_OK):
                return log_dir
            elif os.access(os.path.dirname(log_dir), os.W_OK):
                try:
                    os.makedirs(log_dir)
                    return log_dir
                except:
                    continue



# Generated by Django 4.0.2 on 2022-02-24 19:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0003_migrate_redirects_briefurls'),
    ]

    operations = [
        migrations.AlterField(
            model_name='burluser',
            name='first_name',
            field=models.CharField(blank=True, max_length=150, verbose_name='first name'),
        ),
    ]



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from .models import BurlUser


admin.site.register(BurlUser, UserAdmin)
admin.site.site_header = "burl admin"
admin.site.site_title = "burl admin"
admin.site.index_title = "home"



from django.contrib.auth import get_user_model
from django.conf import settings
from django.contrib.sites.models import Site
from django.test import TestCase

from django_burl.models import BriefURL, BriefURLDomainUser, BriefURLDefaultRedirect


class BurlCoreViewTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        Site.objects.filter(id=1).update(domain="testserver", name="testserver")
        cls.site = Site.objects.get(id=1)
        cls.amy = get_user_model().objects.create_user("amy")
        BriefURLDomainUser.objects.create(
            site=cls.site, user=cls.amy, role=BriefURLDomainUser.Role.ADMIN
        )
        cls.burl = BriefURL.objects.create(
            url="https://arstechnica.com", site=cls.site, user=cls.amy
        )

    def test_root_redirect_settings_default(self):
        response = self.client.get("/")
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response.headers["Location"], settings.DEFAULT_REDIRECT_URL)

    def test_root_redirect_db_default(self):
        BriefURLDefaultRedirect.objects.create(
            site=self.site, url="https://www.nytimes.com"
        )
        response = self.client.get("/")
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response.headers["Location"], "https://www.nytimes.com")

    def test_redirect(self):
        response = self.client.get(f"/{self.burl.burl}/")
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response.headers["Location"], "https://arstechnica.com")

    def test_api_root(self):
        response = self.client.get("/api/")
        self.assertEquals(response.status_code, 200)
        self.assertTrue("v1" in response.json())
        self.assertTrue("v2" in response.json())

    def test_api_v2_root(self):
        response = self.client.get("/api/v2/")
        self.assertEquals(response.status_code, 200)
        self.assertTrue("burls" in response.json())
        self.assertTrue("token" in response.json())

    def test_api_v2_burls(self):
        response = self.client.get("/api/v2/burls/")
        self.assertEquals(response.status_code, 401)
        self.assertEquals(
            response.json()["detail"], "Authentication credentials were not provided."
        )

    def test_api_v2_token_root(self):
        response = self.client.get("/api/v2/token/")
        self.assertEquals(response.status_code, 200)
        self.assertTrue("auth" in response.json())
        self.assertTrue("refresh" in response.json())
        self.assertTrue("verify" in response.json())



from rest_framework.response import Response
from rest_framework.decorators import api_view
from rest_framework.reverse import reverse
from rest_framework_simplejwt.tokens import RefreshToken


@api_view(["GET"])
def root(request, fmt=None):
    return Response(
        {
            "v1": reverse("api_v1:root", request=request, format=fmt),
            "v2": reverse("api_v2:root", request=request, format=fmt),
        }
    )


@api_view(["GET"])
def v1_root(request, fmt=None):
    root_navigation = {
        "redirects": reverse(
            "api_v1:redirects:redirect-list", request=request, format=fmt
        ),
        "token": reverse("api_v1:token_root", request=request, format=fmt),
    }
    return Response(root_navigation)


@api_view(["GET"])
def v2_root(request, fmt=None):
    root_navigation = {
        "burls": reverse("api_v2:burls:burls-list", request=request, format=fmt),
        "token": reverse("api_v2:token_root", request=request, format=fmt),
    }
    return Response(root_navigation)


@api_view(["GET"])
def token_root(request, fmt=None):
    token_navigation = {
        "auth": reverse("api_v1:token_auth", request=request, format=fmt),
        "refresh": reverse("api_v1:token_refresh", request=request, format=fmt),
        "verify": reverse("api_v1:token_verify", request=request, format=fmt),
    }
    return Response(token_navigation)


@api_view(["GET"])
def v2_token_root(request, fmt=None):
    token_navigation = {
        "auth": reverse("api_v2:token_auth", request=request, format=fmt),
        "refresh": reverse("api_v2:token_refresh", request=request, format=fmt),
        "verify": reverse("api_v2:token_verify", request=request, format=fmt),
    }
    return Response(token_navigation)


@api_view(["POST"])
def token_refresh(request):
    token = request.COOKIES.get("burl_refresh_token")
    if token:
        refresh = RefreshToken(str(token))
        access = str(refresh.access_token)
        if access:
            return Response({"access": access}, 200)
        else:
            return Response({"unauthorized"}, 401)
    return Response("unauthorized", 401)


@api_view(["POST"])
def token_refresh_revoke(_request):
    response = Response("ok")
    response.delete_cookie("burl_refresh_token")
    return response



import logging
import os
from datetime import timedelta

from burl.core import utils
from burl.core.conf import config

# NOTE: attempt to group related settings together

# situational awareness settings
SETTINGS_MODULE = os.path.dirname(os.path.abspath(__file__))
CORE_MODULE = os.path.dirname(SETTINGS_MODULE)
MODULE_ROOT = os.path.dirname(CORE_MODULE)
BASE_DIR = MODULE_ROOT
PROJECT_ROOT = os.path.dirname(MODULE_ROOT)
HOME = utils.get_env("HOME")

# general application settings
DEBUG = config.get("app.debug", bool)
HASHID_ALPHABET = config.get("app.hashid_alphabet", str)
BURL_BLACKLIST = config.get("app.burl_blacklist", list)
ROUGH_COUNT_MIN = config.get("admin.rough_count_min", int)

# security related settings
AUTH_USER_MODEL = "core.BurlUser"
SECRET_KEY = config.get("security.secret_key", str)
ALLOWED_HOSTS = config.get("security.allowed_hosts", list)
CORS_ALLOW_CREDENTIALS = True
CORS_ALLOW_ALL_ORIGINS = config.get("security.cors.allow_all_origins", bool)
CORS_ALLOWED_ORIGINS = config.get("security.cors.allowed_origins", list)
CORS_ALLOWED_ORIGIN_REGEXES = config.get("security.cors.allowed_origin_regexes", list)
SIMPLE_JWT = {
    "ACCESS_TOKEN_LIFETIME": timedelta(
        seconds=config.get("security.jwt.access_lifetime", int)
    ),
    "REFRESH_TOKEN_LIFETIME": timedelta(
        seconds=config.get("security.jwt.refresh_lifetime", int)
    ),
}

# localization related settings
LANGUAGE_CODE = "en-us"
TIME_ZONE = config.get("app.time_zone", str)
USE_I18N = True
USE_L10N = True
USE_TZ = True

# http related settings
WSGI_APPLICATION = "burl.core.wsgi.application"
USE_X_FORWARDED_HOST = config.get("http.use_x_forwarded_host", bool)
SECURE_PROXY_SSL_HEADER = (
    config.get("http.secure_proxy_ssl_header_name"),
    config.get("http.secure_proxy_ssl_header_value"),
)

# URLs and URL configurations
ROOT_URLCONF = "burl.core.urls.root"
LOGIN_URL = "/accounts/login/"
LOGOUT_URL = "/accounts/logout/"
LOGIN_REDIRECT_URL = "/api/v2/swagger/"
STATIC_URL = "/static/"
DEFAULT_REDIRECT_URL = config.get("app.default_redirect_url")

# file resources
MEDIA_ROOT = config.get("app.media_root")
STATIC_ROOT = config.get("app.static_root")
STATICFILES_STORAGE = "whitenoise.storage.CompressedManifestStaticFilesStorage"

# rest framework settings
REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework_simplejwt.authentication.JWTAuthentication",
        "rest_framework.authentication.SessionAuthentication",
        "rest_framework.authentication.TokenAuthentication",
    ),
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.LimitOffsetPagination",
    "PAGE_SIZE": config.get("api.page_size"),
    "DEFAULT_PARSER_CLASSES": [
        "rest_framework.parsers.JSONParser",
    ],
    "DEFAULT_FILTER_BACKENDS": ["django_filters.rest_framework.DjangoFilterBackend"],
}
SWAGGER_SETTINGS = {
    "SECURITY_DEFINITIONS": {
        "token": {"type": "apiKey", "in": "header", "name": "Authorization"},
    },
}

# email settings
SENDGRID_API_KEY = config.get("mail.sendgrid_api_key")
if SENDGRID_API_KEY == 1 or SENDGRID_API_KEY == "":
    EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"
else:
    EMAIL_BACKEND = "sendgrid_backend.SendgridBackend"
DEFAULT_FROM_EMAIL = config.get("mail.default_from_email")

# high-level logging configuration (see more detailed configs below)
LOG_DIR = config.get("logging.log_dir")
LOG_LEVELS = {
    "debug": logging.DEBUG,
    "info": logging.INFO,
    "warning": logging.WARNING,
    "error": logging.ERROR,
    "critical": logging.CRITICAL,
}
BURL_LOG_LEVEL = LOG_LEVELS.get(config.get("logging.burl.level"), logging.WARN)
APP_LOG_LEVEL = LOG_LEVELS.get(config.get("logging.app.level"), logging.WARN)

# database configuration
DATABASES = {
    "default": {
        "ENGINE": config.get("db.default.engine", str),
        "NAME": config.get("db.default.name", str),
        "USER": config.get("db.default.user", str),
        "PASSWORD": config.get("db.default.password", str),
        "HOST": config.get("db.default.host", str),
        "PORT": config.get("db.default.port", int),
    },
}

INSTALLED_APPS = [
    "whitenoise.runserver_nostatic",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "burl.core",
    "corsheaders",
    "rest_framework",
    "rest_framework.authtoken",
    "django.contrib.admin",
    "django.contrib.sites",
    "django_filters",
    "django_burl",
]

try:
    import django_extensions

    INSTALLED_APPS.append("django_extensions")
except ImportError:
    pass

MIDDLEWARE = [
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "whitenoise.middleware.WhiteNoiseMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.contrib.sites.middleware.CurrentSiteMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]


AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "verbose": {
            "format": "[%(asctime)s] %(levelname)s [%(name)s:%(lineno)s] %(message)s",
            "datefmt": "%d/%b/%Y %H:%M:%S",
        },
        "simple": {"format": "%(levelname)s %(message)s"},
    },
    "handlers": {
        "file": {
            "level": BURL_LOG_LEVEL,
            "class": "logging.handlers.RotatingFileHandler",
            "filename": os.path.join(LOG_DIR, "burl.log"),
            "maxBytes": 1024 * 1024 * 5,  # 5MiB
            "backupCount": 5,
            "formatter": "verbose",
        },
        "console": {
            "class": "logging.StreamHandler",
            "formatter": "verbose",
            "level": BURL_LOG_LEVEL,
        },
    },
    "loggers": {
        "": {
            "handlers": ["console"],
            "level": APP_LOG_LEVEL,
        },
        "burl": {
            "handlers": ["console"],
            "level": BURL_LOG_LEVEL,
        },
    },
}



from django.db import migrations
from django.apps import apps as global_apps


def forward(apps, _schema_editor):
    try:
        Redirect = apps.get_model("redirects", "Redirect")
    except LookupError:
        return

    BriefURL = apps.get_model("django_burl", "BriefURL")
    BriefURL.objects.bulk_create(
        BriefURL(
            id=redirect.id,
            url=redirect.url,
            burl=redirect.burl,
            description=redirect.description,
            created=redirect.created,
            updated=redirect.updated,
            enabled=redirect.enabled,
            user=redirect.user,
        )
        for redirect in Redirect.objects.all()
    )


class Migration(migrations.Migration):
    operations = [migrations.RunPython(forward, migrations.RunPython.noop)]
    dependencies = [
        ("django_burl", "0001_initial"),
        ("core", "0002_auto_20210508_1204"),
    ]

    if global_apps.is_installed("burl.redirects"):
        dependencies.append(("redirects", "0001_initial"))



import uuid

from django.contrib.auth.models import AbstractUser
from django.db.models import UUIDField


class BurlUser(AbstractUser):
    id = UUIDField(default=uuid.uuid4, primary_key=True, editable=False)



import os

from cfitall.config import ConfigManager
from burl.core import utils

home = os.environ.get("HOME")

config = ConfigManager("burl")

config.set_default("admin.rough_count_min", 1000)

config.set_default("api.page_size", 20)

config.set_default("app.burl_blacklist", ["admin", "api", "static", "media"])
config.set_default("app.debug", False)
config.set_default("app.default_redirect_url", "https://www.wikipedia.org/")
config.set_default(
    "app.hashid_alphabet", "abcdefghjkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ0123456789"
)
config.set_default(
    "app.media_root", os.path.join(home, ".local", "var", "burl", "media")
)
config.set_default("app.time_zone", "America/Los_Angeles")
config.set_default(
    "app.static_root", os.path.join(home, ".local", "share", "burl", "static")
)

config.set_default("db.default.name", "burl")
config.set_default("db.default.user", "burl")
config.set_default("db.default.password", "burl")
config.set_default("db.default.host", "127.0.0.1")
config.set_default("db.default.port", 5432)
config.set_default("db.default.engine", "django.db.backends.postgresql_psycopg2")

config.set_default("http.use_x_forwarded_host", True)
config.set_default("http.secure_proxy_ssl_header_name", "HTTP_X_FORWARDED_PROTO")
config.set_default("http.secure_proxy_ssl_header_value", "http")

config.set_default("logging.app.level", "warn")
config.set_default("logging.burl.level", "info")
config.set_default("logging.log_dir", utils.get_log_dir())

config.set_default("mail.default_from_email", "nobody@burl.test")
config.set_default("mail.sendgrid_api_key", "")

config.set_default("security.allowed_hosts", [])
config.set_default("security.cors.allowed_origins", [])
config.set_default("security.cors.allowed_origin_regexes", [])
config.set_default("security.cors.allow_all_origins", False)
config.set_default("security.jwt.access_lifetime", 600)
config.set_default("security.jwt.refresh_lifetime", 86400)
config.set_default(
    "security.secret_key",
    "jeirainooyieShaequeeng8av9gah6geiv1ooTh6quoo9meireeRayoo6un7xah",
)

config.add_config_path("/etc/burl")
config.add_config_path(os.path.join(home, ".local", "etc", "burl"))
config.read_config()



from django.urls import include, path
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import TemplateView
from rest_framework.schemas import get_schema_view
from rest_framework_simplejwt.views import (
    TokenVerifyView,
)

from burl.core.api import views
from burl.core.api.viewsets import TokenObtainUserPairView
from django_burl.api.v2 import urls as burl_urls

app_name = "api_v2"

urlpatterns = [
    path("", views.v2_root, name="root"),
    path("", include(burl_urls)),
    path("token/", views.v2_token_root, name="token_root"),
    path("token/auth/", TokenObtainUserPairView.as_view(), name="token_auth"),
    path("token/refresh/", csrf_exempt(views.token_refresh), name="token_refresh"),
    path(
        "token/refresh/revoke/", views.token_refresh_revoke, name="token_refresh_revoke"
    ),
    path("token/verify/", TokenVerifyView.as_view(), name="token_verify"),
    path(
        "swagger/",
        TemplateView.as_view(
            template_name="core/api/swagger.html",
            extra_context={"schema_url": "api_v2:openapi_schema"},
        ),
        name="swagger_ui",
    ),
    path(
        "openapi/",
        get_schema_view(
            title="burl api",
            description="rest api for burl",
            version="2",
            url="/api/v2",
            urlconf="burl.core.urls.api_v2",
        ),
        name="openapi_schema",
    ),
]



"""burl URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.0/topics/http/redirects/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.redirects import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.redirects'))
"""

from django.contrib import admin
from django.urls import include, path
from django.contrib.auth import urls as account_urls

from burl.core.api import views as api_views
from django_burl import views as redirect_views
from burl.core.urls import api_v1, api_v2
from burl.core.views import root_redirect

urlpatterns = [
    path("accounts/", include(account_urls)),
    path("admin/", admin.site.urls),
    path("api/", api_views.root, name="api-root"),
    path("api/v1/", include(api_v1, namespace="api_v1")),
    path("api/v2/", include(api_v2, namespace="api_v2")),
    path("<str:burl>/", redirect_views.get_redirect, name="redirect"),
    path("", root_redirect),
]



# Generated by Django 2.2.22 on 2021-05-08 17:04

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='burluser',
            name='id',
            field=models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
        ),
    ]
